<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\Film;
use App\Form\FilmType;
use Doctrine\Common\Collections\Criteria;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Film controller.
 *
 * @Route("film")
 */
class FilmController extends Controller
{
	use FormControllerTrait;

	protected const ENTITY = Film::class;
	protected const FORM = FilmType::class;

	/**
	 * Lists all film entities.
	 *
	 * @Route("/", name="film_index", methods={"GET"})
	 */
	public function indexAction()
	{

		$repo = $this->getDoctrine()->getManager()->getRepository(self::ENTITY);

		$criteria = Criteria::create()
			->where(Criteria::expr()->gt('rollsInCamera', 0))
			->orderBy([
				'filmFormat' => Criteria::ASC,
				'brand' => Criteria::ASC,
				'rollsInCamera' => Criteria::DESC,
				'productLine' => Criteria::ASC,
			]);

		$inCamera = $repo->matching($criteria);

		$notInCamera = $repo->findBy([
			'rollsInCamera' => 0,
		], [
			'brand' => 'ASC',
			'productLine' => 'ASC',
			'filmFormat' => 'ASC',
		]);

		return $this->render('film/index.html.twig', [
			'in_camera' => $inCamera,
			'films' => $notInCamera,
		]);
	}

	/**
	 * Creates a new film entity.
	 *
	 * @Route("/new", name="film_new", methods={"GET", "POST"})
	 */
	public function newAction(Request $request)
	{
		return $this->itemCreate($request, 'film/new.html.twig', 'film', 'film_show');
	}

	/**
	 * Finds and displays a film entity.
	 *
	 * @Route("/{id}", name="film_show", methods={"GET"})
	 */
	public function showAction(Film $film)
	{
		return $this->itemView($film, 'film/show.html.twig', 'film');
	}

	/**
	 * Displays a form to edit an existing film entity.
	 *
	 * @Route("/{id}/edit", name="film_edit", methods={"GET", "POST"})
	 * @throws \LogicException
	 */
	public function editAction(Request $request, Film $film)
	{
		return $this->itemUpdate($request, $film, 'film/edit.html.twig', 'film', 'film_show');
	}

	/**
	 * Deletes a film entity.
	 *
	 * @Route("/{id}", name="film_delete", methods={"DELETE"})
	 * @throws \LogicException
	 */
	public function deleteAction(Request $request, Film $film)
	{
		return $this->itemDelete($request, $film, 'film_index');
	}

	/**
	 * Creates a form to delete a film entity.
	 *
	 * @param Film $film The film entity
	 *
	 * @return \Symfony\Component\Form\FormInterface The form
	 */
	private function createDeleteForm(Film $film): FormInterface
	{
		return $this->buildForm($film, 'film_delete', 'DELETE');
	}
}
